SystemOrganization addCategory: #TinyBlog!
SystemOrganization addCategory: 'TinyBlog-Tests'!

TestCase subclass: #TBMBlogTest
	instanceVariableNames: 'blog first post'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TinyBlog-Tests'!

!TBMBlogTest methodsFor: 'as yet unclassified' stamp: 'FrancoisJarriges 5/10/2016 17:53'!
setUp
	"initialize the tests context by removing all posts in the current blog
	then add a new post and create a second one that is not yet recorded in the blog"

	blog := TBMBlog current.
	blog removeAllBlogPosts.
	first := TBMPost title: 'A title' text: 'A text' category: 'First Category'.
	blog addBlogPost: first.
	post := (TBMPost title: 'Another title' text: 'Another text' category: 'Second Category') beVisible! !

!TBMBlogTest methodsFor: 'as yet unclassified' stamp: 'FrancoisJarriges 5/10/2016 10:38'!
tearDown
	"reset the current blog at the end of the tests"

	TBMBlog reset! !

!TBMBlogTest methodsFor: 'tests-action' stamp: 'FrancoisJarriges 5/10/2016 17:53'!
testAddBlogPost
	"add the already created post (at set up) then verify that it is actually recorded in the blog"

	blog addBlogPost: post.
	self assert: blog size equals: 2! !

!TBMBlogTest methodsFor: 'tests-action' stamp: 'FrancoisJarriges 5/10/2016 17:53'!
testAllBlogPosts
	"add the already created post to the blog then verify it is actually recorded 
	by getting the posts entire collection and checking its size"

	blog addBlogPost: post.
	self assert: blog allBlogPosts size equals: 2! !

!TBMBlogTest methodsFor: 'tests-action' stamp: 'FrancoisJarriges 5/10/2016 11:39'!
testAllBlogPostsFromCategory
	"verify that the first post is actually recorded as in first category
	by checking the number of corresponding posts in the current blog collection"

	self assert: (blog allBlogPostsFromCategory: 'First Category') size equals: 1! !

!TBMBlogTest methodsFor: 'tests-action' stamp: 'FrancoisJarriges 5/10/2016 17:53'!
testAllCategories
	"add the post already created and declared in second category to the current blog collection
	then create and add again a new post defined in the same second category
	then collect all the categories defined among these posts
	and finally check there is two categories actually defined"

	blog addBlogPost: post.
	blog addBlogPost: (TBMPost title: 'Another title again' text: 'Another text again' category: 'Second Category').
	self assert: blog allCategories size equals: 2! !

!TBMBlogTest methodsFor: 'tests-action' stamp: 'FrancoisJarriges 5/10/2016 17:53'!
testAllCategoriesFromVisibleBlogPosts
	"add the post already created and declared as visible in second category to the current blog collection
	then collect all the categories defined among the visible posts
	and finally check there is only one category actually defined"

	blog addBlogPost: post.
	self assert: blog allCategoriesFromVisibleBlogPosts size equals: 1! !

!TBMBlogTest methodsFor: 'tests-action' stamp: 'FrancoisJarriges 5/10/2016 17:59'!
testAllNotVisibleBlogPosts
	"verify that the first post is actually recorded as not visible in the collection 
	by checking the number of  unvisible posts"

	self assert: blog allNotVisibleBlogPosts size equals: 1! !

!TBMBlogTest methodsFor: 'tests-action' stamp: 'FrancoisJarriges 5/10/2016 17:53'!
testAllVisibleBlogPosts
	"add the post already created and declared as visible to the blog 
	then verify it is actually recorded as visible in the collection 
	by checking the number of visible posts"

	blog addBlogPost: post.
	self assert: blog allVisibleBlogPosts size equals: 1! !

!TBMBlogTest methodsFor: 'tests-action' stamp: 'FrancoisJarriges 5/10/2016 17:53'!
testAllVisibleBlogPostsFromCategory
	"add the post already created and declared as visible in second category to the current blog collection
	then verify there is no visible post in the first category
	and finally check there is only one visible post in this second category"

	blog addBlogPost: post.
	self assert: (blog allVisibleBlogPostsFromCategory: 'First Category') size equals: 0.
	self assert: (blog allVisibleBlogPostsFromCategory: 'Second Category') size equals: 1! !

!TBMBlogTest methodsFor: 'tests-action' stamp: 'FrancoisJarriges 5/10/2016 17:53'!
testCategoryBeNotVisible
	"add the post already created as visible and declared in second category to the current blog collection
	then make this second category unvisible
	and finally check the number of unvisible posts in the current blog collection"

	blog addBlogPost: post.
	blog categoryBeNotVisible: 'Second Category'.
	self assert: blog allVisibleBlogPosts size equals: 0! !

!TBMBlogTest methodsFor: 'tests-action' stamp: 'FrancoisJarriges 5/10/2016 17:53'!
testCategoryBeVisible
	"add the post already created as visible and declared in second category to the current blog collection
	then make the first category visible as well
	and finally check the number of visible posts in the current blog collection"

	blog addBlogPost: post.
	blog categoryBeVisible: 'First Category'.
	self assert: blog allVisibleBlogPosts size equals: 2! !

!TBMBlogTest methodsFor: 'tests-action' stamp: 'FrancoisJarriges 5/10/2016 17:53'!
testCategoryRenameFromTo
	"add the post already created and declared in second category to the current blog collection
	then rename the first category by giving it the name of the second one
	and finally check the number of remaining categories in the current blog collection"

	blog addBlogPost: post.
	blog categoryRenameFrom: 'First Category' to: 'Second Category'.
	self assert: blog allCategories size equals: 1! !

!TBMBlogTest methodsFor: 'tests-action' stamp: 'FrancoisJarriges 5/10/2016 15:12'!
testRemoveAllBlogPosts
	"remove all posts from the current blog then verify it is actually empty"

	blog removeAllBlogPosts.
	self assert: blog size equals: 0! !

!TBMBlogTest methodsFor: 'tests-action' stamp: 'FrancoisJarriges 5/10/2016 17:53'!
testRemoveAllBlogPostsFromCategory
	"add the post already created and declared as visible in second category to the current blog collection
	then remove all posts recorded in the first category from the current blog 
	and verify that it remains only the second post by checking the number of posts to one"

	blog addBlogPost: post.
	blog removeAllBlogPostsFromCategory: 'First Category'.
	self assert: blog size equals: 1! !

!TBMBlogTest methodsFor: 'tests-action' stamp: 'FrancoisJarriges 5/10/2016 10:59'!
testSize
	"verify the blog size after tests setup"

	self assert: blog size equals: 1! !

!TBMBlogTest methodsFor: 'tests-action' stamp: 'FrancoisJarriges 5/10/2016 12:09'!
testUnclassifiedBlogPosts
	"verify the unclassified post control feature 
	by checking there is no unclassified post in the current blog collection"

	self assert: (blog allBlogPosts select: [ :eachPost | eachPost isUnclassified ]) size equals: 0! !

Object subclass: #TBMBlog
	instanceVariableNames: 'posts'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TinyBlog'!

TBMBlog class
	instanceVariableNames: 'uniqueInstance'!

!TBMBlog commentStamp: 'FrancoisJarriges 5/10/2016 15:46' prior: 0!
I represent a unique blog  in the Tiny Blog (FUN MOOC - Programmation immersive en Pharo)

I provide methods on  the class side to manage the unique instance of myself 

I provide methods to record posts to myself and manage them through the criterias defined in their class

Public API and Key Messages :

aPost := TBMPost title: 'A title'  text: 'A text'  category: 'A category' .

blog :=  TBMBlog current.		"unique instance of TBMBlog"
blog writeBlogPost:  aPost.	"add a post to the blog"

"Get selected posts in an OrderedCollection of posts :"

allPosts :=  blog allBlogPosts.
allPostsFromACategory :=	blog allBlogPostsFromCategory:  'A category'.
allVisiblePosts := blog allVisibleBlogPosts.
allVisiblePostsFromACategory := blog allVisibleBlogPostsFromCategory:  'A category'.

"Get  an actual Set  of  all categories among the selected posts :"

allCategories := blog allCategories.
allCategoriesFromVisiblePosts := blog allCategoriesFromVisibleBlogPosts.

"Get the actual number of posts :"
numberOfPosts := blog size.

"Clear the selected posts from the blog :"
blog removeAllBlogPostsFromCategory: 'A category'.
blog removeAllPosts .

Internal Representation and Key Implementation Points.

    Instance Variables
	posts:	<OrderedCollection>
		
     Class Instance Variables 
	uniqueInstance: 	<TBMBlog>
	
    Implementation Points!

TBMBlog class
	instanceVariableNames: 'uniqueInstance'!

!TBMBlog class methodsFor: 'as yet unclassified' stamp: 'FrancoisJarriges 5/10/2016 18:08'!
createDemoPosts
"TBMBlog createDemoPosts"
	
self current
addBlogPost: ((TBPost title: 'Welcome in TinyBlog' text:
'TinyBlog is a small blog engine made with Pharo.' category:
'TinyBlog') visible: true);
addBlogPost: ((TBPost title: 'Report Pharo Sprint' text:
'Friday, June 12 there was a Pharo sprint / Moose dojo. It was a
nice event with more than 15 motivated sprinters. With the help
of candies, cakes and chocolate, huge work has been done'
category: 'Pharo') visible: true);
addBlogPost: ((TBPost title: 'Brick on top of Bloc -
Preview' text: 'We are happy to announce the first preview
version of Brick, a new widget set created from scratch on top
of Bloc. Brick is being developed primarily by Alex Syrel
(together with Alain Plantec, Andrei Chis and myself), and the
work is sponsored by ESUG.
Brick is part of the Glamorous Toolkit effort and will provide
the basis for the new versions of the development tools.'
category: 'Pharo') visible: true);
addBlogPost: ((TBPost title: 'The sad story of unclassified
blog posts' text: 'So sad that I can read this.') visible: true);
addBlogPost: ((TBPost title: 'Working with Pharo on the
Raspberry Pi' text: 'Hardware is getting cheaper and many new
small devices like the famous Raspberry Pi provide new
computation power that was one once only available on regular
desktop computers.' category: 'Pharo') visible: true)! !

!TBMBlog class methodsFor: 'as yet unclassified' stamp: 'FrancoisJarriges 5/10/2016 13:26'!
current
	"answer the unique instance of the TBMBlog and create it if it doesn't exist yet"
	
	^ uniqueInstance ifNil: [ uniqueInstance := self new ]! !

!TBMBlog class methodsFor: 'initializing' stamp: 'FrancoisJarriges 5/10/2016 13:24'!
initialize
	"reset the TBMBlog instance singleton when TBMBlog class is loaded "

	self reset ! !

TBMBlog initialize!

!TBMBlog class methodsFor: 'as yet unclassified' stamp: 'FrancoisJarriges 5/10/2016 10:06'!
reset
	uniqueInstance := nil! !

!TBMBlog methodsFor: 'action' stamp: 'FrancoisJarriges 5/10/2016 17:53'!
addBlogPost: aTBMPost
	"write the given post in the current blog database"

	posts add: aTBMPost! !

!TBMBlog methodsFor: 'action' stamp: 'FrancoisJarriges 5/10/2016 11:26'!
allBlogPosts
	"answer the entire posts collection defined in the current blog"

	^ posts! !

!TBMBlog methodsFor: 'action' stamp: 'FrancoisJarriges 5/10/2016 15:33'!
allBlogPostsFromCategory: aString 
	"answer a selection of all the posts classified in the given category from the current blog"
	
	^ posts select: [ :eachPost | eachPost category = aString ]! !

!TBMBlog methodsFor: 'action' stamp: 'FrancoisJarriges 5/10/2016 13:11'!
allCategories
	"answer a set of all the posts categories defined in the current blog"
	
	^ (posts collect: [ :eachPost | eachPost category ]) asSet! !

!TBMBlog methodsFor: 'action' stamp: 'FrancoisJarriges 5/10/2016 15:34'!
allCategoriesFromVisibleBlogPosts
	"answer a set of all the categories defined among visible posts from the current blog"
	
	^ ((posts select: [ :eachPost | eachPost isVisible ]) 
		collect: [ :eachVisiblePost | eachVisiblePost category ]) asSet! !

!TBMBlog methodsFor: 'action' stamp: 'FrancoisJarriges 5/10/2016 18:01'!
allNotVisibleBlogPosts
	"answer a selection of all the unvisible posts from the current blog"
	
	^ posts select: [ :eachPost | eachPost isVisible not]! !

!TBMBlog methodsFor: 'action' stamp: 'FrancoisJarriges 5/10/2016 15:34'!
allVisibleBlogPosts
	"answer a selection of all the visible posts from the current blog"
	
	^ posts select: [ :eachPost | eachPost isVisible ]! !

!TBMBlog methodsFor: 'action' stamp: 'FrancoisJarriges 5/10/2016 15:35'!
allVisibleBlogPostsFromCategory: aString 
	"answer a selection of all the visible posts in the given category from the current blog"
	
	^ posts select: [ :eachPost | 
		eachPost category = aString 
		and: [ 
		eachPost isVisible ] ]! !

!TBMBlog methodsFor: 'action' stamp: 'FrancoisJarriges 5/10/2016 17:46'!
categoryBeNotVisible: aString 
	"make each post unvisible within the given category identified as aString
	answer a selection of all the posts classified in the given category from the current blog"
	
	| postsFromSelectedCategory |
	
	postsFromSelectedCategory := self allBlogPostsFromCategory: aString.
	postsFromSelectedCategory do: [ :eachPostFromSelectedCategory | 
		eachPostFromSelectedCategory beNotVisible ].
	^ postsFromSelectedCategory! !

!TBMBlog methodsFor: 'action' stamp: 'FrancoisJarriges 5/10/2016 17:44'!
categoryBeVisible: aString 
	"make each post visible within the given category identified as aString
	answer a selection of all the posts classified in the given category from the current blog"
	
	| postsFromSelectedCategory |
	
	postsFromSelectedCategory := self allBlogPostsFromCategory: aString.
	postsFromSelectedCategory do: [ :eachPostFromSelectedCategory | 
		eachPostFromSelectedCategory beVisible ].
	^ postsFromSelectedCategory! !

!TBMBlog methodsFor: 'action' stamp: 'FrancoisJarriges 5/10/2016 17:30'!
categoryRenameFrom: actualString to: newString
	"rename the actual category identified as actualString by giving it newString as new name
	answer a selection of all the posts classified in the given category from the current blog"

	| postsFromSelectedCategory |
	
	postsFromSelectedCategory := self allBlogPostsFromCategory: actualString.
	postsFromSelectedCategory do: [ :eachPostFromSelectedCategory | 
		eachPostFromSelectedCategory category: newString ].
	^ postsFromSelectedCategory! !

!TBMBlog methodsFor: 'initializing' stamp: 'FrancoisJarriges 5/10/2016 10:04'!
initialize

	super initialize.
	posts := OrderedCollection new! !

!TBMBlog methodsFor: 'action' stamp: 'FrancoisJarriges 5/10/2016 15:12'!
removeAllBlogPosts
	"remove all posts by creating a new empty collection for the current blog"

	posts := OrderedCollection new! !

!TBMBlog methodsFor: 'action' stamp: 'FrancoisJarriges 5/10/2016 15:45'!
removeAllBlogPostsFromCategory: aString
	"remove all the posts classified in the given category from the current blog"

	posts	removeAllSuchThat: [ :eachPost | eachPost category = aString ]! !

!TBMBlog methodsFor: 'as yet unclassified' stamp: 'FrancoisJarriges 5/10/2016 10:57'!
size
	"answer the size of the posts collection in the current blog"

	^ posts size! !

Object subclass: #TBMPost
	instanceVariableNames: 'title text date category visible'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TinyBlog'!

!TBMPost commentStamp: 'FrancoisJarriges 5/10/2016 14:09' prior: 0!
I represent a post in the Tiny Blog (FUN MOOC - Programmation immersive en Pharo)

I provide methods to create and manage  a post


Public API and Key Messages :

post1 := TBMPost title: 'test post 1'	text: 'created unclassified and unvisible' .
post2 := TBMPost title: 'test post 2'  text: 'created unvisible'  category: 'Testing' .

post1 isUnclassified ifTrue:  [post1 category: 'Testing'] .
post1 isVisible  ifFalse:  [post1 beVisible ] .

post1 beNotVisible; category: self class unclassifiedTag .



Internal Representation and Key Implementation Points.

    Instance Variables
	category:	<String>
	date:		<Date>
	text:		<Text>
	title:		<String>
	visible:		<Boolean>


    Implementation Points!

!TBMPost class methodsFor: 'as yet unclassified' stamp: 'FrancoisJarriges 5/5/2016 10:47'!
title: aTitle text: aText

	^ self new 
		title: aTitle;
		text: aText;
		yourself ! !

!TBMPost class methodsFor: 'as yet unclassified' stamp: 'FrancoisJarriges 5/5/2016 10:49'!
title: aTitle text: aText category: aCategory

	^ (self title: aTitle text: aText) 
		category: aCategory;
		yourself ! !

!TBMPost class methodsFor: 'as yet unclassified' stamp: 'FrancoisJarriges 5/4/2016 17:08'!
unclassifiedTag

	^ 'Unclassified'! !

!TBMPost methodsFor: 'visibility' stamp: 'FrancoisJarriges 5/4/2016 17:04'!
beNotVisible

	self visible: false! !

!TBMPost methodsFor: 'visibility' stamp: 'FrancoisJarriges 5/4/2016 17:03'!
beVisible

	self visible: true! !

!TBMPost methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/4/2016 17:01'!
category

	^ category! !

!TBMPost methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/9/2016 08:36'!
category: aString

	category := aString! !

!TBMPost methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/4/2016 17:00'!
date

	^ date! !

!TBMPost methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/9/2016 08:54'!
date: aDate

	date := aDate! !

!TBMPost methodsFor: 'initialization' stamp: 'FrancoisJarriges 5/10/2016 14:11'!
initialize

	super initialize.
	self category: self class unclassifiedTag.
	self date: Date today.
	self beNotVisible ! !

!TBMPost methodsFor: 'testing' stamp: 'FrancoisJarriges 5/5/2016 10:57'!
isUnclassified

	^ self category = self class unclassifiedTag ! !

!TBMPost methodsFor: 'testing' stamp: 'FrancoisJarriges 5/4/2016 17:04'!
isVisible

	^ self visible! !

!TBMPost methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/4/2016 16:59'!
text

	^ text! !

!TBMPost methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/9/2016 08:20'!
text: aText

	text := aText ! !

!TBMPost methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/4/2016 16:58'!
title

	^ title! !

!TBMPost methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/9/2016 08:31'!
title: aString

	title := aString ! !

!TBMPost methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/4/2016 17:01'!
visible

	^ visible! !

!TBMPost methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/5/2016 10:34'!
visible: aBoolean

	visible := aBoolean  ! !

Object subclass: #TBPost
	instanceVariableNames: 'category date text title visible'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TinyBlog'!

!TBPost commentStamp: 'LucFabresse 3/18/2016 10:01' prior: 0!
I represent blog posts.
Inspect the code below :

TBPost
	title: 'Welcome in TinyBlog'
	text: 'TinyBlog is a small blog engine made with Pharo.' 
	category: 'TinyBlog'

!

!TBPost class methodsFor: 'instance creation' stamp: 'StephaneDucasse 12/4/2015 14:44'!
title: aTitle text: aText 
	^ self new
		title: aTitle;
		text: aText;
		yourself! !

!TBPost class methodsFor: 'instance creation' stamp: 'StephaneDucasse 12/4/2015 14:45'!
title: aTitle text: aText category: aCategory
	^ (self title: aTitle text: aText)
			category: aCategory;
			yourself! !

!TBPost class methodsFor: 'constants' stamp: 'StephaneDucasse 12/4/2015 14:36'!
unclassifiedTag
	^ 'Unclassified'! !

!TBPost methodsFor: 'visibility' stamp: 'StephaneDucasse 12/4/2015 14:35'!
beVisible
   self visible: true! !

!TBPost methodsFor: 'accessing' stamp: 'StephaneDucasse 12/4/2015 14:32'!
category
	^ category! !

!TBPost methodsFor: 'accessing' stamp: 'StephaneDucasse 12/4/2015 14:33'!
category: anObject
	category := anObject
! !

!TBPost methodsFor: 'accessing' stamp: 'StephaneDucasse 12/4/2015 14:33'!
date
	^ date! !

!TBPost methodsFor: 'accessing' stamp: 'StephaneDucasse 12/4/2015 14:33'!
date: anObject
	date := anObject! !

!TBPost methodsFor: 'initialize' stamp: 'StephaneDucasse 12/4/2015 14:39'!
initialize
	self category: self class unclassifiedTag.
	self date: Date today.
	self notVisible! !

!TBPost methodsFor: 'testing' stamp: 'StephaneDucasse 12/4/2015 14:48'!
isUnclassified
   ^ self category = self class unclassifiedTag! !

!TBPost methodsFor: 'testing' stamp: 'StephaneDucasse 12/4/2015 14:35'!
isVisible
   ^ self visible! !

!TBPost methodsFor: 'visibility' stamp: 'StephaneDucasse 12/4/2015 14:35'!
notVisible
   self visible: false! !

!TBPost methodsFor: 'accessing' stamp: 'StephaneDucasse 12/4/2015 14:33'!
text
	^ text! !

!TBPost methodsFor: 'accessing' stamp: 'StephaneDucasse 12/4/2015 14:33'!
text: anObject
	text := anObject! !

!TBPost methodsFor: 'accessing' stamp: 'StephaneDucasse 12/4/2015 14:33'!
title
	^ title! !

!TBPost methodsFor: 'accessing' stamp: 'StephaneDucasse 12/4/2015 14:33'!
title: anObject
	title := anObject! !

!TBPost methodsFor: 'accessing' stamp: 'StephaneDucasse 12/4/2015 14:33'!
visible
	^ visible! !

!TBPost methodsFor: 'accessing' stamp: 'StephaneDucasse 12/4/2015 14:33'!
visible: anObject
	visible := anObject! !

TBMBlog initialize!
